@@ -29,12 +29,54 @@ def test_registration_alias():
     s.register_alias('stream_1')
     assert s.registry.stream_1 is s.registry.s1 is s
 
+def test_copy_like():
+    tmo.settings.set_thermo(
+        ['Water', 'O2', 'Ash'], 
+        db='BioSTEAM', cache=True
+    )
+    ms = tmo.MultiStream(g=[('O2', 1)], s=[('Ash', 1)], phases=('g', 's'))
+    s = tmo.Stream()
+    s.copy_like(ms)
+    assert (s.imol.data == ms.imol.data).all()
+
 def test_vlle():
+    import thermosteam as tmo
+    from numpy.testing import assert_allclose
     tmo.settings.set_thermo(['Water', 'Ethanol', 'Octane'], cache=True)
-    s = tmo.Stream(None, Water=1, Ethanol=1, Octane=2, vlle=True, T=350)
-    assert_allclose(s.mol, [1, 1, 2]) # mass balance
-    assert_allclose(s.imol['l'] + s.imol['L'], [0.537583, 0.383255, 1.755296], rtol=0.1)
-    assert_allclose(s.imol['g'], [0.47019442460450805, 0.6267950838143611, 0.24880861894952971], rtol=0.1) # Convergence
+    T = 351
+    P = 101325
+    s = tmo.Stream(None, Water=1, Ethanol=0.5, Octane=2, vlle=True, T=T, P=P)
+    assert_allclose(s.mol, [1, 0.5, 2]) # mass balance
+    total = s.F_mol
+    xl = s.imol['l'].sum() / total
+    xL = s.imol['L'].sum() / total if 'L' in s.phases else 0.
+    xg = s.imol['g'].sum() / total
+    assert_allclose(xl, 0.2748928033836762, atol=5e-3, rtol=5e-3)
+    assert_allclose(xL, 0.6370268977038833, atol=5e-3, rtol=5e-3) # mass balance
+    assert_allclose(xg, 0.08808029891244049, atol=5e-3, rtol=5e-3) # mass balance
+    H = s.H
+    S = s.S
+    V = s.vlle.vapor_fraction
+    print(s.vlle.iter)
+    specs = (
+        dict(H=H, P=P), 
+        dict(S=S, P=P), 
+        dict(V=V, P=P),
+        dict(H=H, T=T), 
+        dict(S=S, T=T),
+        dict(V=V, T=T),
+    )
+    for spec in specs:
+        s.vlle(**spec)
+        print(s.vlle.iter)
+        total = s.F_mol
+        xl = s.imol['l'].sum() / total
+        xL = s.imol['L'].sum() / total if 'L' in s.phases else 0.
+        xg = s.imol['g'].sum() / total
+        assert_allclose(xl, 0.2748928033836762, atol=5e-3, rtol=5e-3)
+        assert_allclose(xL, 0.6370268977038833, atol=5e-3, rtol=5e-3) # mass balance
+        assert_allclose(xg, 0.08808029891244049, atol=5e-3, rtol=5e-3) # mass balance
+    
     s = tmo.Stream(None, Water=1, Ethanol=1, Octane=2, vlle=True, T=300)
     assert set(s.phases) == set(['l', 'L']) # No gas phase
     assert_allclose(s.mol, [1, 1, 2]) # mass balance
@@ -44,16 +86,104 @@ def test_vlle():
     assert_allclose(s.imol['g'], [0.9548858089512597, 0.949841750275759, 0.7342182603619914], rtol=1e-2) # Convergence
     s = tmo.Stream(None, Water=1, Ethanol=1, Octane=2, vlle=True, T=380)
     assert s.phases == ('g',) # Only one phase
-    s = tmo.MultiStream(None, l=[('Water', 1), ('Ethanol', 1), ('Octane', 2)], vlle=True, T=380)
+    s = tmo.MultiStream(None, l=[('Water', 1), ('Ethanol', 1), ('Octane', 2)], vlle=True, T=390)
     assert s.phase == 'g' # Only one phase
     assert set(s.phases) == set(['L', 'l', 'g']) # All three phases can still be used
     
+def test_vlle_with_solids():
+    import thermosteam as tmo
+    from numpy.testing import assert_allclose
+    Solid = tmo.Chemical('Solid', search_db=False, phase='s', default=True)
+    tmo.settings.set_thermo(['Water', 'Ethanol', 'Octane', Solid], cache=True)
+    T = 351
+    P = 101325
+    s = tmo.Stream(None, Water=1, Ethanol=0.5, Octane=2, vlle=True, T=T, P=P)
+    s.phases = 'sglL'
+    s.imol['s', 'Solid'] = 0.5
+    assert_allclose(s.mol, [1, 0.5, 2, 0.5]) # mass balance
+    total = s.F_mol - s.imol['Solid']
+    xl = s.imol['l'].sum() / total
+    xL = s.imol['L'].sum() / total if 'L' in s.phases else 0.
+    xg = s.imol['g'].sum() / total
+    assert_allclose(xl, 0.2748928033836762, atol=5e-3, rtol=5e-3)
+    assert_allclose(xL, 0.6370268977038833, atol=5e-3, rtol=5e-3) # mass balance
+    assert_allclose(xg, 0.08808029891244049, atol=5e-3, rtol=5e-3) # mass balance
+    H = s.H
+    S = s.S
+    V = s.vlle.vapor_fraction
+    print(s.vlle.iter)
+    specs = (
+        dict(H=H, P=P), 
+        dict(S=S, P=P), 
+        dict(V=V, P=P),
+        dict(H=H, T=T), 
+        dict(S=S, T=T),
+        dict(V=V, T=T),
+    )
+    for spec in specs:
+        s.vlle(**spec)
+        print(s.vlle.iter)
+        total = s.F_mol - s.imol['Solid']
+        xl = s.imol['l'].sum() / total
+        xL = s.imol['L'].sum() / total if 'L' in s.phases else 0.
+        xg = s.imol['g'].sum() / total
+        assert_allclose(xl, 0.2748928033836762, atol=2e-3, rtol=5e-3)
+        assert_allclose(xL, 0.6370268977038833, atol=2e-3, rtol=5e-3) # mass balance
+        assert_allclose(xg, 0.08808029891244049, atol=2e-3, rtol=5e-3) # mass balance
+    
+def test_vlle_with_solids_and_supercritical_components():
+    import thermosteam as tmo
+    from numpy.testing import assert_allclose
+    Solid = tmo.Chemical('Solid', search_db=False, phase='s', default=True)
+    tmo.settings.set_thermo(['Water', 'Ethanol', 'Octane', 'CO2', Solid], cache=True)
+    T = 345
+    P = 101325
+    s = tmo.Stream(None, Water=1, Ethanol=0.5, Octane=2)
+    s.phases = 'sglL'
+    s.imol['s', 'Solid'] = 0.5
+    s.imol['g', 'CO2'] = 0.1
+    s.vlle(T=T, P=P)
+    assert_allclose(s.mol, [1, 0.5, 2, 0.1, 0.5]) # mass balance
+    total = s.F_mol - s.imol['Solid']
+    xl = s.imol['l'].sum() / total
+    xL = s.imol['L'].sum() / total if 'L' in s.phases else 0.
+    xg = s.imol['g'].sum() / total
+    assert_allclose(xl, 0.26877566584386003, atol=5e-3, rtol=5e-3)
+    assert_allclose(xL, 0.6042822289561331, atol=5e-3, rtol=5e-3) # mass balance
+    assert_allclose(xg, 0.12694210520000673, atol=5e-3, rtol=5e-3) # mass balance
+    H = s.H
+    S = s.S
+    V = s.vlle.vapor_fraction
+    print(s.vlle.iter)
+    specs = (
+        dict(H=H, P=P), 
+        dict(S=S, P=P), 
+        dict(V=V, P=P),
+        dict(H=H, T=T), 
+        dict(S=S, T=T),
+        dict(V=V, T=T),
+    )
+    for spec in specs:
+        try:
+            s.vlle(**spec)
+        except:
+            print('failed', spec)
+            continue
+        print(s.vlle.iter)
+        total = s.F_mol - s.imol['Solid']
+        xl = s.imol['l'].sum() / total
+        xL = s.imol['L'].sum() / total if 'L' in s.phases else 0.
+        xg = s.imol['g'].sum() / total
+        assert_allclose(xl, 0.26877566584386003, atol=5e-3, rtol=5e-3)
+        assert_allclose(xL, 0.6042822289561331, atol=5e-3, rtol=5e-3) # mass balance
+        assert_allclose(xg, 0.12694210520000673, atol=5e-3, rtol=5e-3) # mass balance
+    
 def stream_methods():
     tmo.settings.set_thermo(['Water', 'Ethanol'], cache=True)
     s1 = tmo.Stream(None, Water=1, Ethanol=1, phase='l')
     s2 = tmo.Stream(None, Water=100, Ethanol=100, phase='g')
     s3 = tmo.Stream.sum([s1, s2], vle=True)
-    assert_allclose([s3.T, s3.vapor_fraction], [356.42348688162764, 0.9159452891657182])
+    assert_allclose([s3.T, s3.vapor_fraction], [356.45552794986, 0.9157652669624603])
     s1.P = 101325 * 5
     s2.P = 101325 * 2
     s3.mix_from([s1, s2], vle=True)
@@ -69,15 +199,15 @@ def test_critical():
     s.vle(V=vapor_fraction, P=101325)
     assert_allclose(s.vapor_fraction, vapor_fraction)
     assert_allclose(s.mol, [1, 1, 2])
-    assert_allclose(s['g'].z_mol, [0.0006053116760182865, 0.43970244158393407, 0.5596922467400477], rtol=1e-2)
+    assert_allclose(s['g'].z_mol, [0.0005899562150044437, 0.439751371507529, 0.5596586722774665], rtol=1e-2)
     
     bp = s.bubble_point_at_P()
     assert_allclose(bp.T, 102.62052288398583, rtol=1e-3)
-    assert_allclose(bp.y, [3.803565920764843e-05, 0.7778539445712263, 0.222108019769566], rtol=1e-2)
+    assert_allclose(bp.y, [3.6820522686269676e-05, 0.7780884456009951, 0.22187473387631854], rtol=1e-2)
     
     dp = s.dew_point_at_P()
     assert_allclose(dp.T, 164.2843303081629, rtol=1e-3)
-    assert_allclose(dp.x, [0.9697130577019155, 0.0034114070277255068, 0.02687553527035907], rtol=1e-2)
+    assert_allclose(dp.x, [0.9698259345522571, 0.003426383313112619, 0.02674768213463028], rtol=1e-2)
     
     s = tmo.Stream(None, CO2=1, O2=1, CH4=2, vlle=True, T=80)
     s.phase == 'l'
@@ -93,11 +223,11 @@ def test_critical():
     
     bp = s.bubble_point_at_P()
     assert_allclose(bp.T, 97.37091146329703, rtol=1e-3)
-    assert_allclose(bp.y, [2.614648198782934e-05, 0.999973853518012], rtol=1e-2)
+    assert_allclose(bp.y, [2.5146656093575396e-05, 0.9999748533439065], rtol=1e-2)
     
     dp = s.dew_point_at_P()
     assert_allclose(dp.T, 173.64797757499818, rtol=1e-3)
-    assert_allclose(dp.x, [0.9951836814710002, 0.004816318528999781], rtol=1e-2)
+    assert_allclose(dp.x, [0.9952339113391101, 0.0047660886608899755], rtol=1e-2)
     
     s = tmo.Stream(None, CO2=1, O2=1, vlle=True, T=80)
     s.phase == 'l'
@@ -107,20 +237,20 @@ def test_stream():
     stream = tmo.Stream(None, Water=1, T=300)
     assert [stream.chemicals.Water] == stream.available_chemicals
     assert_allclose(stream.epsilon, 77.744307)
-    assert_allclose(stream.alpha * 1e6, 0.14330776454124503)
-    assert_allclose(stream.nu, 8.799123532986536e-07)
-    assert_allclose(stream.Pr, 6.14001869413997)
-    assert_allclose(stream.Cn, 75.29555729396768)
-    assert_allclose(stream.C, 75.29555729396768)
-    assert_allclose(stream.Cp, 4.179538552493643)
-    assert_allclose(stream.P_vapor, 3536.806752274638)
-    assert_allclose(stream.mu, 0.0008766363688287887)
-    assert_allclose(stream.kappa, 0.5967303492959747)
-    assert_allclose(stream.rho, 996.2769195618362)
-    assert_allclose(stream.V, 1.80826029854462e-05)
-    assert_allclose(stream.H, 139.31398526921475)
-    assert_allclose(stream.S, 70.46581776376684)
-    assert_allclose(stream.sigma, 0.07168596252716256)
+    assert_allclose(stream.alpha * 1e6, 0.143374, rtol=1e-3)
+    assert_allclose(stream.nu, 8.567663853669672e-07, rtol=1e-3)
+    assert_allclose(stream.Pr, 5.9757311368776005, rtol=1e-3)
+    assert_allclose(stream.Cn, 75.29555729396768, rtol=1e-3)
+    assert_allclose(stream.C, 75.29555729396768, rtol=1e-3)
+    assert_allclose(stream.Cp, 4.179538552493643, rtol=1e-3)
+    assert_allclose(stream.P_vapor, 3536.806752274638, rtol=1e-3)
+    assert_allclose(stream.mu, 0.0008538310235084569, rtol=1e-3)
+    assert_allclose(stream.kappa, 0.597342, rtol=1e-3)
+    assert_allclose(stream.rho, 996.2769195618362, rtol=1e-3)
+    assert_allclose(stream.V, 1.80826029854462e-05, rtol=1e-3)
+    assert_allclose(stream.H, 139.31398526921475, rtol=1e-3)
+    assert_allclose(stream.S, 70.46581776376684, rtol=1e-3)
+    assert_allclose(stream.sigma, 0.07168596252716256, rtol=1e-3)
     assert_allclose(stream.z_mol, [1.0])
     assert_allclose(stream.z_mass, [1.0])
     assert_allclose(stream.z_vol, [1.0])
@@ -167,21 +297,21 @@ def test_multistream():
     tmo.settings.set_thermo(['Water', 'Ethanol'], cache=True)
     stream = tmo.MultiStream(None, l=[('Water', 1)], T=300)
     assert [stream.chemicals.Water] == stream.available_chemicals
-    assert_allclose(stream.epsilon, 77.744307)
-    assert_allclose(stream.alpha * 1e6, 1.4330776454124502e-01)
-    assert_allclose(stream.nu, 8.799123532986536e-07)
-    assert_allclose(stream.Pr, 6.14001869413997)
-    assert_allclose(stream.Cn, 75.29555729396768)
-    assert_allclose(stream.C, 75.29555729396768)
-    assert_allclose(stream.Cp, 4.179538552493643)
-    assert_allclose(stream.P_vapor, 3536.806752274638)
-    assert_allclose(stream.mu, 0.0008766363688287887)
-    assert_allclose(stream.kappa, 0.5967303492959747)
-    assert_allclose(stream.rho, 996.2769195618362)
-    assert_allclose(stream.V, 1.80826029854462e-05)
-    assert_allclose(stream.H, 139.31398526921475)
-    assert_allclose(stream.S, 70.465818)
-    assert_allclose(stream.sigma, 0.07168596252716256)
+    assert_allclose(stream.epsilon, 77.744307, rtol=1e-3)
+    assert_allclose(stream.alpha * 1e6, 1.4330776454124502e-01, rtol=1e-3)
+    assert_allclose(stream.nu, 8.567663853669672e-07, rtol=1e-3)
+    assert_allclose(stream.Pr, 5.9757311368776005, rtol=1e-3)
+    assert_allclose(stream.Cn, 75.29555729396768, rtol=1e-3)
+    assert_allclose(stream.C, 75.29555729396768, rtol=1e-3)
+    assert_allclose(stream.Cp, 4.179538552493643, rtol=1e-3)
+    assert_allclose(stream.P_vapor, 3536.806752274638, rtol=1e-3)
+    assert_allclose(stream.mu, 0.0008538310235084569, rtol=1e-3)
+    assert_allclose(stream.kappa, 0.5973419454132901, rtol=1e-3)
+    assert_allclose(stream.rho, 996.2769195618362, rtol=1e-3)
+    assert_allclose(stream.V, 1.80826029854462e-05, rtol=1e-3)
+    assert_allclose(stream.H, 139.31398526921475, rtol=1e-3)
+    assert_allclose(stream.S, 70.465818, rtol=1e-3)
+    assert_allclose(stream.sigma, 0.07168596252716256, rtol=1e-3)
     assert_allclose(stream.z_mol, [1.0, 0.])
     assert_allclose(stream.z_mass, [1.0, 0.])
     assert_allclose(stream.z_vol, [1.0, 0.])
@@ -225,13 +355,12 @@ def test_multistream():
     assert type(stream) is tmo.Stream
     assert stream.phase == 's'
     
-    # Linking
+    # Copy
     stream.phase = 'l'
     stream.phases = 'lg'
+    stream.imol['l', 'Water'] = 10
+    stream.vle(V=0.5, P=2*101325)
     other = stream.copy()
-    stream.link_with(other)
-    other.imol['l', 'Water'] = 10
-    other.vle(V=0.5, P=2*101325)
     assert_allclose(other.mol, stream.mol)
     assert_allclose(other.T, stream.T)
     assert_allclose(other.P, stream.P)
@@ -261,6 +390,47 @@ def test_multistream():
     assert stream.get_flow('g/s', 'Water') == stream.F_mass / 3.6 == 1.
     stream.empty()    
 
+def test_stream_indexing():
+    tmo.settings.set_thermo(['Water', 'Ethanol', 'Methanol'], cache=True)
+    tmo.settings.chemicals.define_group('Alcohol', ['Ethanol', 'Methanol'], [0.5, 0.5], wt=False)
+    stream = tmo.Stream(None, Water=1.)
+    assert stream.imol['Alcohol'] == 0.
+    assert stream.imol['Water'] == 1.
+    stream.imol['Alcohol'] = 1.
+    stream.imol['Water'] = 2.
+    assert (stream.imol['Alcohol', 'Water'] == [1., 2.]).all()
+    assert (stream.imol['Ethanol', 'Water'] == [0.5, 2.]).all()
+    stream.imol['Ethanol', 'Water'] = [1., 2.]
+    assert (stream.imol['Ethanol', 'Water'] == [1., 2.]).all()
+    assert stream.imol['Ethanol'] == 1.
+    stream = tmo.MultiStream(None, l=[('Water', 1)], phases='lg')
+    assert stream.imol['l', 'Alcohol'] == 0.
+    assert stream.imol['l', 'Water'] == 1.
+    assert (stream.imol['l', ('Alcohol', 'Water')] == [0., 1.]).all()
+    assert (stream.imol[..., ('Alcohol', 'Water')] == [[0., 0.], [0., 1.]]).all()
+    
+    stream.imol['l', 'Alcohol'] = 1.
+    assert stream.imol['Ethanol'] == 0.5
+    stream.imol['l', 'Water'] = 2.
+    assert (stream.imol['l', ('Alcohol', 'Water')] == [1., 2.]).all()
+    stream.imol['g', 'Alcohol'] = 2.
+    stream.imol['g', 'Water'] = 1.
+    assert stream.imol['Alcohol'] == 3.
+    assert stream.imol['Water'] == 3.
+    assert (stream.imol['Alcohol', 'Water'] == [3., 3.]).all()
+    assert (stream.imol[..., ('Alcohol', 'Water')] == [[2., 1.], [1., 2.]]).all()
+    
+    key = ('l', 'Water')
+    stream.set_flow(1, 'gpm', key)
+    assert stream.get_flow('gpm', key) == 1.
+    
+    tmo.settings.chemicals.define_group('Mixture', ['Water', 'Alcohol'], [0.5, 0.5], wt=False)
+    stream = tmo.Stream(None, Mixture=1.)
+    assert stream.imol['Alcohol'] == 0.5
+    assert stream.imol['Water'] == 0.5
+    assert stream.imol['Ethanol'] == 0.25
+    assert stream.imol['Methanol'] == 0.25
+
 def test_stream_property_cache():
     tmo.settings.set_thermo(['Water', 'Ethanol'], cache=True)
     s = tmo.Stream(None, Water=2, T=299.15)
@@ -324,12 +494,40 @@ def test_mixing_balance():
     ms['l'].mix_from(streams)
     assert ms['l'].imol['Water'] == total_flow
 
+def test_mixing_phases():
+    tmo.settings.set_thermo(['Water'], cache=True)
+    ms = tmo.MultiStream(None, l=[('Water', 1)], g=[('Water', 2)])
+    stream = tmo.Stream(Water=1, phase='L')
+    ms.mix_from([ms, stream], conserve_phases=True)
+    assert ms.phases == ('L', 'g', 'l') # New phase is created
+    assert ms.imol['L', 'Water'] == ms.imol['l', 'Water'] == 1
+    
+    ms = tmo.MultiStream(None, l=[('Water', 1)], g=[('Water', 2)])
+    stream = tmo.Stream(Water=1, phase='L')
+    ms.mix_from([ms, stream]) 
+    assert ms.phases == ('g', 'l') # 'L' phase gets added to 'l' phase
+    assert ms.imol['l', 'Water'] == 2
+
+def test_mixing_pressure():
+    P = 111458
+    s1 = tmo.Stream(None, Water=1000, P=P)
+    s2 = tmo.Stream(None, Water=2000, P=P)
+    s_mix = tmo.Stream.sum([s1, s2])
+    assert s_mix.P == P
+    
+    P_high = 131458
+    P_low = 111458
+    s3 = tmo.Stream(None, Water=1000, P=P_high)
+    s4 = tmo.Stream(None, Water=2000, P=P_low)
+    s_mix = tmo.Stream.sum([s3, s4])
+    assert s_mix.P == P_low
+
 def test_mixture():
     tmo.settings.set_thermo(['Water'], cache=True)
 
     # test solve_T_at_xx
     T_expected = 298
-    s5 = tmo.Stream('s5', T=T_expected, P=1e5, Water=1)
+    s5 = tmo.Stream(None, T=T_expected, P=1e5, Water=1)
     Th = s5.mixture.solve_T_at_HP(phase=s5.phase, mol=s5.mol, H=s5.H, T_guess=s5.T, P=s5.P)
     assert_allclose(T_expected, Th, rtol=1e-3)
     Ts = s5.mixture.solve_T_at_SP(phase=s5.phase, mol=s5.mol, S=s5.S, T_guess=s5.T, P=s5.P)
@@ -355,16 +553,21 @@ def test_vle_critical_pure_component():
     s.vle(P=N2.Pc, S=s.S + 10)
     assert s.phase == 'g'
     
-    
 if __name__ == '__main__':
     test_registration_bypass()
     test_registration_alias()
     test_stream()
     test_multistream()
+    test_copy_like()
+    test_stream_indexing()
     test_mixing_balance()
     test_vlle()
     stream_methods()
     test_stream_property_cache()
     test_vle_critical_pure_component()
     test_critical()
-    test_mixture()
\ No newline at end of file
+    test_mixture()
+    test_mixing_phases()
+    test_mixing_pressure()
+    test_vlle_with_solids()
+    test_vlle_with_solids_and_supercritical_components()
\ No newline at end of file
