@@ -31,7 +31,7 @@
 #     update_module(chemicals, numba)
 # use_numba_chemicals()
 # del use_numba_chemicals
-__version__ = "0.32.0"
+__version__ = "0.52.0"
 
 from . import thermo
 del thermo
@@ -45,21 +45,23 @@ from . import (
     units_of_measure,
     separations,
     functors,
+    phenomenode,
 )
 from .utils import docround
+from .network import *
 from ._chemical_data import ChemicalData
 from ._chemical import Chemical
-from ._chemicals import Chemicals, CompiledChemicals
+from ._chemicals import Chemicals, CompiledChemicals, ChemicalDraft, ChemicalsOutline
 from ._thermal_condition import ThermalCondition
-from . import mixture
-from .mixture import Mixture
 from ._thermo import Thermo, IdealThermo
 from ._settings import settings, ProcessSettings
 from ._thermo_data import ThermoData
 from . import (
     indexer,
     reaction,
-    equilibrium
+    equilibrium,
+    mixture,
+    network,
 )
 from ._stream import Stream
 from ._heat_and_power import Heat, Power
@@ -67,13 +69,18 @@ from ._multi_stream import MultiStream
 from .base import functor
 from .reaction import *
 from .equilibrium import * 
+from .mixture import *
+from ._preferences import preferences
+from .phenomenode import *
 
-__all__ = ('Chemical', 'ChemicalData', 'Chemicals', 'CompiledChemicals', 'Mixture', 'Thermo', 
-           'IdealThermo', 'Stream', 'MultiStream', 'Heat', 'Power', 'ThermalCondition', 'ProcessSettings',
-           'mixture', 'ThermoData', *reaction.__all__, *equilibrium.__all__, 
+__all__ = ('Chemical', 'ChemicalData', 'Chemicals', 'CompiledChemicals', 
+           'ChemicalDraft', 'ChemicalsOutline', 'Thermo', 'IdealThermo', 
+           'Stream', 'MultiStream', 'Heat', 'Power', 'ThermalCondition', 'ProcessSettings',
+           'mixture', 'ThermoData', *reaction.__all__, *equilibrium.__all__,  *mixture.__all__,
+           *network.__all__, 'preferences',
            'indexer', 'settings', 'functor', 'functors', 'chemicals', 'base', 
            'equilibrium', 'units_of_measure', 'exceptions', 'functional', 
-           'reaction', 'constants', 'utils', 'separations')
+           'reaction', 'constants', 'utils', 'separations', *phenomenode.__all__)
 
 # Set number of digits displayed
 import numpy as np
@@ -81,7 +88,7 @@ import pandas as pd
 np.set_printoptions(suppress=False)
 np.set_printoptions(precision=3) 
 pd.options.display.float_format = '{:.3g}'.format
-pd.set_option('display.max_rows', 35)
+pd.set_option('display.max_rows', 50)
 pd.set_option('display.max_columns', 10)
 pd.set_option('max_colwidth', 35)
 pd.set_option('display.width', 160)
