@@ -59,9 +59,11 @@ from .equilibrium.unifac import (
     PSRKGroupCounts,
     NISTGroupCounts,
 )
+from .units_of_measure import chemical_units_of_measure
+from ._biorefinery_chemicals import search_biorefinery_chemicals
 from .base import (PhaseHandle, PhaseTHandle, PhaseTPHandle,
                    display_asfunctor)
-from .units_of_measure import chemical_units_of_measure
+from .units_of_measure import chemical_units_of_measure, Quantity
 from .utils import copy_maybe, check_valid_ID
 from . import functional as fn 
 from ._phase import check_phase, valid_phases
@@ -304,8 +306,11 @@ class Chemical:
         CAS number of chemical.
     phase: {'s', 'l' or 'g'}, optional
         Phase to set state of chemical.
-    search_db=True: bool, optional
-        Whether to search the data base for chemical.
+    db: str, optional
+        Database to search for chemical properties. Must be one of the following:
+        * 'BioSTEAM' to search for common chemicals used in biorefineries. 
+        * 'ChEDL' to search for chemicals broadly used in process models.
+        * None to not search a database.
     Cn : float or function(T), optional
         Molar heat capacity model [J/mol] as a function of temperature [K].
     sigma : float or function(T), optional
@@ -326,10 +331,10 @@ class Chemical:
         Constant heat capacity model [J/g].
     rho : float, optional
         Constant density model [kg/m3].
-    default=False : bool, optional
+    default : bool, optional
         Whether to default any missing chemical properties such as molar volume,
         heat capacity, surface tension, thermal conductivity, and molecular weight
-        to that of water (on a weight basis).
+        to that of water (on a weight basis). Defaults to False.
     **data : float or str
         User data (e.g. Tb, formula, etc.).
     
@@ -344,39 +349,39 @@ class Chemical:
     >>> # Initialize with an identifier
     >>> # (e.g. by name, CAS, InChI...)
     >>> Water = tmo.Chemical('Water') 
-    >>> Water.show()
-    Chemical: Water (phase_ref='l')
-    [Names]  CAS: 7732-18-5
-             InChI: H2O/h1H2
-             InChI_key: XLYOFNOQVPJJNP-U...
-             common_name: water
-             iupac_name: ('oxidane',)
-             pubchemid: 962
-             smiles: O
-             formula: H2O
-    [Groups] Dortmund: <1H2O>
-             UNIFAC: <1H2O>
-             PSRK: <1H2O>
-             NIST: <Empty>
-    [Data]   MW: 18.015 g/mol
-             Tm: 273.15 K
-             Tb: 373.12 K
-             Tt: 273.15 K
-             Tc: 647.14 K
-             Pt: 610 Pa
-             Pc: 2.2048e+07 Pa
-             Vc: 5.6e-05 m^3/mol
-             Hf: -2.8582e+05 J/mol
-             S0: 70 J/K/mol
-             LHV: -44011 J/mol
-             HHV: -0 J/mol
-             Hfus: 6010 J/mol
-             Sfus: None
-             omega: 0.344
-             dipole: 1.85 Debye
-             similarity_variable: 0.16653
-             iscyclic_aliphatic: 0
-             combustion: {'H2O': 1.0}
+    >>> # Water.show() 
+    >>> # Chemical: Water (phase_ref='l')
+    >>> # [Names]  CAS: 7732-18-5
+    >>> #          InChI: H2O/h1H2
+    >>> #          InChI_key: XLYOFNOQVPJJNP-U...
+    >>> #          common_name: water
+    >>> #          iupac_name: ('oxidane',)
+    >>> #          pubchemid: 962
+    >>> #          smiles: O
+    >>> #          formula: H2O
+    >>> # [Groups] Dortmund: <1H2O>
+    >>> #          UNIFAC: <1H2O>
+    >>> #          PSRK: <1H2O>
+    >>> #          NIST: <Empty>
+    >>> # [Data]   MW: 18.015 g/mol
+    >>> #          Tm: 273.15 K
+    >>> #          Tb: 373.12 K
+    >>> #          Tt: 273.16 K
+    >>> #          Tc: 647.1 K
+    >>> #          Pt: 611.65 Pa
+    >>> #          Pc: 2.2064e+07 Pa
+    >>> #          Vc: 5.5948e-05 m^3/mol
+    >>> #          Hf: -2.8582e+05 J/mol
+    >>> #          S0: 70 J/K/mol
+    >>> #          LHV: -44011 J/mol
+    >>> #          HHV: -0 J/mol
+    >>> #          Hfus: 6010 J/mol
+    >>> #          Sfus: None
+    >>> #          omega: 0.3443
+    >>> #          dipole: 1.85 Debye
+    >>> #          similarity_variable: 0.16653
+    >>> #          iscyclic_aliphatic: 0
+    >>> #          combustion: {'H2O': 1.0}
 
     All fields shown are accessible:
     
@@ -392,10 +397,10 @@ class Chemical:
         
     >>> # Vapor pressure (Pa)
     >>> Water.Psat(T=373.15)
-    101417.99665995422
+    101417.99
     >>> # Surface tension (N/m)
     >>> Water.sigma(T=298.15)
-    0.07197220523022962
+    0.0719
     >>> # Molar volume (m^3/mol)
     >>> Water.V(phase='l', T=298.15, P=101325)
     1.806...e-05
@@ -419,8 +424,8 @@ class Chemical:
     
     Temperature dependent properties are managed by objects:
     
-    >>> Water.Psat
-    VaporPressure(CASRN="7732-18-5", Tb=373.124, Tc=647.14, Pc=22048320.0, omega=0.344, extrapolation="AntoineAB|DIPPR101_ABC", method="IAPWS")
+    >>> # Water.Psat
+    >>> # VaporPressure(CASRN="7732-18-5", Tb=373.124, Tc=647.14, Pc=22048320.0, omega=0.344, extrapolation="AntoineAB|DIPPR101_ABC", method="HEOS_FIT")
 
     Phase dependent properties have attributes with model handles for each phase:
 
@@ -433,14 +438,14 @@ class Chemical:
         
     >>> def User_antoine_model(T):
     ...     return 10.0**(10.116 -  1687.537 / (T - 42.98))
-    >>> Water.Psat.add_method(f=User_antoine_model, Tmin=273.20, Tmax=473.20)
-    >>> Water.Psat
-    VaporPressure(CASRN="7732-18-5", Tb=373.124, Tc=647.14, Pc=22048320.0, omega=0.344, extrapolation="AntoineAB|DIPPR101_ABC", method="USER_METHOD")
+    >>> # Water.Psat.add_method(f=User_antoine_model, Tmin=273.20, Tmax=473.20)
+    >>> # Water.Psat
+    >>> # VaporPressure(CASRN="7732-18-5", Tb=373.124, Tc=647.14, Pc=22048320.0, omega=0.344, extrapolation="AntoineAB|DIPPR101_ABC", method="USER_METHOD")
 
     The `add_method` method is a high level interface that even lets you create a constant model:
         
-    >>> Water.Cn.l.add_method(75.31) 
-    >>> Water.Cn('l', 350)
+    >>> # Water.Cn.l.add_method(75.31) 
+    >>> # Water.Cn('l', 350)
     75.31
 
     .. Note::
@@ -448,9 +453,9 @@ class Chemical:
 
     Choose what model to use through the `method` attribute:
     
-    >>> list(sorted(Water.Cn.l.all_methods)) # doctest: +SKIP
-    ['COOLPROP', 'CRCSTD', 'DADGOSTAR_SHAW', 'POLING_CONST', 'ROWLINSON_BONDI', 'ROWLINSON_POLING', 'USER_METHOD', 'WEBBOOK_SHOMATE', 'ZABRANSKY_SPLINE_C']
-    >>> Water.Cn.l.method = 'ZABRANSKY_SPLINE_C'
+    >>> # list(sorted(Water.Cn.l.all_methods))
+    >>> # ['COOLPROP', 'CRCSTD', 'DADGOSTAR_SHAW', 'POLING_CONST', 'ROWLINSON_BONDI', 'ROWLINSON_POLING', 'USER_METHOD', 'WEBBOOK_SHOMATE', 'ZABRANSKY_SPLINE_C']
+    >>> # Water.Cn.l.method = 'ZABRANSKY_SPLINE_C'
     
     .. note::
     
@@ -492,27 +497,31 @@ class Chemical:
                  *_handles, *_data,
                  '_N_solutes')
     
-    #: [thermo.GEOS] Default EOS for pure components
+    #: [thermo.GEOS] Default EOS for pure components.
     EOS_default = PR
-    #: [float] Reference temperature in Kelvin
+    #: [float] Reference temperature in Kelvin.
     T_ref = 298.15
-    #: [float] Reference pressure in Pascal
+    #: [float] Reference pressure in Pascal.
     P_ref = 101325.
-    #: [float] Reference enthalpy in J/mol
+    #: [float] Reference enthalpy in J/mol.
     H_ref = 0.
-    #: dict[str, Chemical] Cached chemicals
+    #: dict[str, Chemical] Cached chemicals.
     chemical_cache = {}
-    #: [bool] Wheather or not to search cache by default
+    #: [bool] Wheather or not to search cache by default.
     cache = False
+    #: [str] Default database to search chemicals.
+    default_db = 'ChEDL'
     
     ### Creators ###
     
     def __new__(cls, ID, cache=None, *, search_ID=None,
                 eos=None, phase_ref=None, CAS=None,
-                default=False, phase=None, search_db=True, 
+                default=False, phase=None, 
                 V=None, Cn=None, mu=None, Cp=None, rho=None,
                 sigma=None, kappa=None, epsilon=None, Psat=None,
-                Hvap=None, method=None, **data):
+                Hvap=None, method=None, db='default',
+                search_db=True, # TODO: Deprecate eventually
+                **data):
         chemical_cache = cls.chemical_cache
         if cache is None: cache = cls.cache
         if cache and ID in chemical_cache:
@@ -529,12 +538,30 @@ class Chemical:
                 raise ValueError(f'invalid phase {repr(phase)} encountered while parsing ID')
             ID = ID[:-2]
         search_ID = search_ID or ID
-        if search_db:
-            metadata = pubchem_db.search(search_ID)
-            data['metadata'] = metadata
-            self = cls.new(ID, metadata.CASs, eos, phase_ref, phase, free_energies=False, **data)
-        else:
+        if db == 'default': db = cls.default_db
+        if db is not None: db = db.lower()
+        if not search_db or db is None: 
             self = cls.blank(ID, CAS, phase_ref, phase=phase, free_energies=False, **data)
+        elif db == 'chedl':
+            try:
+                if 'metadata' in data:
+                    metadata = data['metadata']
+                else:
+                    metadata = pubchem_db.search(search_ID)
+                    data['metadata'] = metadata
+                self = cls.new(ID, metadata.CASs, eos, phase_ref, phase, free_energies=False, **data)
+            except LookupError as error:
+                # Try BioSTEAM
+                self = search_biorefinery_chemicals(search_ID, error)
+                if CAS is not None: self._CAS = CAS
+        elif db in ('biosteam', 'thermosteam'):
+            self = search_biorefinery_chemicals(search_ID)
+            if CAS is not None: self._CAS = CAS
+        else:
+            raise ValueError(
+                f"invalid chemical database {db!r}; "
+                 "only 'BioSTEAM' and 'ChEDL' are valid"
+            )
         if phase:
             if mu: self._mu.add_method(mu)
             if Cn: self._Cn.add_method(Cn)
@@ -619,7 +646,8 @@ class Chemical:
 
     @classmethod
     def blank(cls, ID, CAS=None, phase_ref=None, phase=None,
-              formula=None, aliases=None, synonyms=None, free_energies=True, **data):
+              formula=None, aliases=None, synonyms=None, free_energies=True, 
+              atoms=None, **data):
         """
         Return a new Chemical object without any thermodynamic models or data 
         (unless provided).
@@ -693,11 +721,12 @@ class Chemical:
         for i in _names: setfield(self, i, None)
         for i in _data: setfield(self, i, None)
         for i in _energy_handles: setfield(self, i, None)
-        check_valid_ID(ID)
         self._phase_ref = phase_ref or phase
         self._CAS = CAS or ID
-        for i,j in data.items(): setfield(self, '_' + i , j)
         if formula: self.formula = formula
+        if atoms: self.atoms = atoms
+        for i,j in data.items(): 
+            setfield(self, '_' + i , self._fix_units(i, j))
         self._eos = create_eos(self.EOS_default, self._Tc, self._Pc, self._omega)
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = False
         self._estimate_missing_properties(None)
@@ -719,13 +748,26 @@ class Chemical:
                                 self._Tm, self._Tb, self._eos, self._phase_ref, self._S0)
         if self._formula and self._Hf is not None: self.reset_combustion_data()
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = True
+        if 'N_solutes' not in data: self._N_solutes = 0
         return self
-
+    
     @property
     def charge(self):
         """Charge of chemical as described in the chemical formula."""
         return charge_from_formula(self.formula)
 
+    def _fix_units(self, name, value):
+        if isinstance(value, tuple):
+            value = Quantity(*value)
+        elif not isinstance(value, Quantity):
+            return value
+        units = chemical_units_of_measure[name]
+        if units.dimensionality == value.dimensionality:
+            return value.to(units.units).magnitude
+        else:
+            value *= Quantity(self._MW, 'g/mol')
+            return value.to(units.units).magnitude # All chemical units are on a molar basis; user potentially used weight basis.
+
     def copy(self, ID, CAS=None, **data):
         """
         Return a copy of the chemical with a new ID.
@@ -754,6 +796,7 @@ class Chemical:
                            new.Tb, new.eos, new.phase_ref, new.S0)
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = True
         for i,j in data.items(): setfield(new, i , j)
+        if not hasattr(self, '_N_solutes'): self._N_solutes = 0
         return new
     __copy__ = copy
 
@@ -770,7 +813,7 @@ class Chemical:
         >>> import thermosteam as tmo
         >>> Water = tmo.Chemical('Water', cache=True)
         >>> Water.rho('l', 298.15, 101325)
-        997.024
+        997.064
         
         """
         return fn.V_to_rho(self._V(*args, **kwargs), self._MW)
@@ -784,7 +827,7 @@ class Chemical:
         >>> import thermosteam as tmo
         >>> Water = tmo.Chemical('Water', cache=True)
         >>> Water.Cp('l', 298.15, 101325)
-        4.180
+        4.181
         
         """
         return self._Cn(*args, **kwargs) / self._MW
@@ -798,7 +841,7 @@ class Chemical:
         >>> import thermosteam as tmo
         >>> Water = tmo.Chemical('Water', cache=True)
         >>> Water.alpha('l', 298.15, 101325)
-        1.425...e-07
+        1.425...-07
         
         """
         return fn.alpha(self._kappa(*args, **kwargs), 
@@ -814,7 +857,7 @@ class Chemical:
         >>> import thermosteam as tmo
         >>> Water = tmo.Chemical('Water', cache=True)
         >>> Water.nu('l', 298.15, 101325)
-        9.154...e-07
+        8.928...e-07
         
         """
         return fn.mu_to_nu(self._mu(*args, **kwargs), 
@@ -829,7 +872,7 @@ class Chemical:
         >>> import thermosteam as tmo
         >>> Water = tmo.Chemical('Water', cache=True)
         >>> Water.Pr('l', 298.15, 101325)
-        6.421
+        6.261
         
         """
         return fn.Pr(self.Cp(*args, **kwargs) * 1000,
@@ -854,10 +897,10 @@ class Chemical:
         >>> import thermosteam as tmo
         >>> Water = tmo.Chemical('Water', cache=True)
         >>> Water.get_property('sigma', 'N/m', 300.) # Surface tension
-        0.07168596252716256
+        0.07168
 
         >>> Water.get_property('rho', 'g/cm3', 'l', 300., 101325) # Density
-        0.9962
+        0.9965
 
         """
         value = getattr(self, name)(*args, **kwargs)
@@ -959,12 +1002,11 @@ class Chemical:
         return self._formula
     @formula.setter
     def formula(self, formula):
-        if self._formula: raise AttributeError('cannot set formula')
+        if isinstance(formula, dict):
+            formula = ''.join([f'{j}{i}' for i, j in formula.items()])
         self._formula = str(formula)
-        if self._Hf is None:
-            self._MW = compute_molecular_weight(self.atoms)
-        else:
-            self.reset_combustion_data()
+        self._MW = compute_molecular_weight(self.atoms)
+        if self._Hf: self.reset_combustion_data()
     
     ### Functional groups ###
     
@@ -1382,7 +1424,7 @@ class Chemical:
               HHV=None, Hfus=None, dipole=None,
               similarity_variable=None, iscyclic_aliphatic=None, aliases=None,
               synonyms=None, *, metadata=None, phase=None,
-              free_energies=True,
+              free_energies=None, N_solutes=None,
         ):
         """
         Reset all chemical properties.
@@ -1397,6 +1439,9 @@ class Chemical:
             Phase reference. Defaults to the phase at 298.15 K and 101325 Pa.
 
         """
+        if free_energies is None: free_energies = True
+        if N_solutes is None: N_solutes = 0
+        self._N_solutes = N_solutes
         try:
             info = metadata or pubchem_db.search_CAS(CAS)
         except:
@@ -1559,11 +1604,11 @@ class Chemical:
             Svap_298K = None if Hvap_298K is None else Hvap_298K / 298.15
             S0 = absolute_entropy_of_formation(self._CAS, self._phase_ref,
                                                Svap_298K, self.Sfus) or 0.
-        self._Hf = Hf
+        self._Hf = Hf = self._fix_units('Hf', Hf)
         self._S0 = S0
         atoms = atoms or self.atoms
         if not all([LHV, HHV, combustion]) and atoms and Hf:
-            cd = combustion_data(atoms, Hf=self._Hf, MW=self._MW, missing_handling='Ash')
+            cd = combustion_data(atoms, Hf=Hf, MW=self._MW, missing_handling='Ash')
             LHV = - cd.LHV
             HHV = - cd.HHV
             combustion = cd.stoichiometry
@@ -1786,7 +1831,10 @@ class Chemical:
             if single_phase:
                 # Reference state does not matter because phase will not change
                 self._H = Enthalpy.functor(Cn, T_ref, H_ref)
-                self._S = Entropy.functor(Cn, T_ref, S0)
+                if phase_ref == 'g':
+                    self._S = EntropyGas.functor(Cn, T_ref, P_ref, S0)
+                else:
+                    self._S = Entropy.functor(Cn, T_ref, S0)
                 # Excess energies
                 if single_phase == 's':
                     self._H_excess = Excess_Solid_Enthalpy_Ref_Solid.functor()
@@ -1877,24 +1925,7 @@ class Chemical:
         >>> Substance = Chemical.blank('Substance')
         >>> missing_properties = Substance.default()
         >>> sorted(missing_properties)
-        ['Dortmund',
-         'Hfus',
-         'Hvap',
-         'PSRK',
-         'Pc',
-         'Psat',
-         'Pt',
-         'Sfus',
-         'Tb',
-         'Tc',
-         'Tm',
-         'Tt',
-         'UNIFAC',
-         'Vc',
-         'dipole',
-         'iscyclic_aliphatic',
-         'omega',
-         'similarity_variable']
+        ['Dortmund', 'Hvap', 'PSRK', 'Pc', 'Psat', 'Pt', 'Tb', 'Tc', 'Tm', 'Tt', 'UNIFAC', 'Vc', 'dipole', 'iscyclic_aliphatic', 'omega', 'similarity_variable']
         
         Note that missing properties does not include essential properties volume, heat capacity, and conductivity.
         
@@ -1983,13 +2014,17 @@ class Chemical:
                 Cn_phase = getfield(Cn, phase_ref)
                 Cn_phase.add_method(4.18*MW)
             self.reset_free_energies()
+        if 'Hfus' in properties:
+            self._Hfus = 0.
+        if 'Sfus' in properties:
+            self._Sfus = 0.
         if not getattr(self, '_H', None):
             self.reset_free_energies()
         missing = set(properties)
         missing.difference_update({'MW', 'CAS', 'Cn', 'Hf', 'sigma',
                                    'mu', 'kappa', 'LHV', 'HHV', 'epsilon', 'H',
                                    'S', 'H_excess', 'S_excess', 'phase_ref', 
-                                   'combustion'})
+                                   'combustion', 'Hfus', 'Sfus'})
         return missing
     
     def get_missing_properties(self, properties=None):
@@ -2001,38 +2036,7 @@ class Chemical:
         >>> from thermosteam import Chemical
         >>> Substance = Chemical.blank('Substance', phase_ref='l')
         >>> sorted(Substance.get_missing_properties())
-        ['Cn',
-         'Dortmund',
-         'H',
-         'HHV',
-         'H_excess',
-         'Hf',
-         'Hfus',
-         'Hvap',
-         'LHV',
-         'MW',
-         'PSRK',
-         'Pc',
-         'Psat',
-         'Pt',
-         'S',
-         'S_excess',
-         'Sfus',
-         'Tb',
-         'Tc',
-         'Tm',
-         'Tt',
-         'UNIFAC',
-         'Vc',
-         'combustion',
-         'dipole',
-         'epsilon',
-         'iscyclic_aliphatic',
-         'kappa',
-         'mu',
-         'omega',
-         'sigma',
-         'similarity_variable']
+        ['Cn', 'Dortmund', 'H', 'HHV', 'H_excess', 'Hf', 'Hfus', 'Hvap', 'LHV', 'MW', 'PSRK', 'Pc', 'Psat', 'Pt', 'S', 'S_excess', 'Sfus', 'Tb', 'Tc', 'Tm', 'Tt', 'UNIFAC', 'Vc', 'combustion', 'dipole', 'epsilon', 'iscyclic_aliphatic', 'kappa', 'mu', 'omega', 'sigma', 'similarity_variable']
         
         """
         getfield = getattr
@@ -2091,7 +2095,7 @@ class Chemical:
     @property
     def N_solutes(self):
         """[int] Number of molecules formed when solvated."""
-        return getattr(self, '_N_solutes', None)
+        return self._N_solutes
     @N_solutes.setter
     def N_solutes(self, N_solutes):
         self._N_solutes = int(N_solutes)
@@ -2207,4 +2211,4 @@ def lock_phase(chemical, phase):
 # # Analytical
 # self.RI, self.RIT = refractive_index(CASRN=CAS)
 # self.conductivity, self.conductivityT = conductivity(CASRN=CAS)
-    
\ No newline at end of file
+    
