@@ -11,15 +11,13 @@ from __future__ import annotations
 from ._thermo import Thermo
 from ._stream import Stream
 from ._thermal_condition import ThermalCondition
-from .indexer import MolarFlowIndexer
+from .indexer import MolarFlowIndexer, nonzeros, parent_indexer
 from ._phase import phase_tuple
 from . import equilibrium as eq
 from . import utils
-from .indexer import nonzeros
 import numpy as np
 from typing import TYPE_CHECKING
 if TYPE_CHECKING:
-    import thermosteam as tmo
     from numpy.typing import NDArray
     from typing import Optional, Sequence, Dict, Tuple
 
@@ -36,6 +34,7 @@ def get_phase_fraction(stream, phases):
             phase_fraction += imol[phase].sum() 
     return phase_fraction / F_mol
 
+@utils.registered_franchise(Stream)
 class MultiStream(Stream):
     """
     Create a MultiStream object that defines material flow rates for multiple
@@ -85,9 +84,9 @@ class MultiStream(Stream):
     ...                      l=[('Water', 20), ('Ethanol', 10)], units='kg/hr')
     >>> s1.show(flow='kg/hr') # Use the show method to select units of display
     MultiStream: s1
-     phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-     flow (kg/hr): (l) Water    20
-                       Ethanol  10
+    phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+    flow (kg/hr): (l) Water    20
+                      Ethanol  10
     
     The temperature and pressure are stored as attributes:
     
@@ -113,7 +112,7 @@ class MultiStream(Stream):
          Ethanol   0.2171
     >>> # Index a single chemical in the liquid phase
     >>> s1.imol['l', 'Water']
-    1.1101687012358397
+    1.1101
     >>> # Index multiple chemicals in the liquid phase
     >>> s1.imol['l', ('Ethanol', 'Water')]
     array([0.217, 1.11 ])
@@ -179,14 +178,14 @@ class MultiStream(Stream):
     
     >>> s1['l'].show()
     Stream: 
-     phase: 'l', T: 365 K, P: 101325 Pa
-     flow (kmol/hr): Water    0.616
-                     Ethanol  0.0235
+    phase: 'l', T: 365 K, P: 101325 Pa
+    flow (kmol/hr): Water    0.617
+                    Ethanol  0.0238
     >>> s1['g'].show()
     Stream: 
-     phase: 'g', T: 365 K, P: 101325 Pa
-     flow (kmol/hr): Water    0.494
-                     Ethanol  0.194
+    phase: 'g', T: 365 K, P: 101325 Pa
+    flow (kmol/hr): Water    0.493
+                    Ethanol  0.193
     
     Note that the phase cannot be changed:
     
@@ -250,8 +249,30 @@ class MultiStream(Stream):
         self._sink = self._source = None
         self.reset_cache()
         self._register(ID)
-        self._user_equilibrium = None
-        if vlle: self.vlle(T, P)
+        if vlle: self.vlle(T=T, P=P)
+        
+    @classmethod
+    def from_streams(cls, streams, thermo=None):
+        if not streams: raise ValueError('at least one stream must be passed')
+        self = cls.__new__(cls)
+        self._streams = streams_by_phase = {i.phase: i for i in streams}
+        phases = phase_tuple(streams_by_phase)
+        N_streams = len(streams)
+        if len(phases) != N_streams: raise ValueError('each stream must have a different phase')
+        base, *others = streams
+        self.characterization_factors = {}
+        self._thermal_condition = base._thermal_condition
+        for i in others: i._thermal_condition = base._thermal_condition
+        self._load_thermo(thermo or base.thermo)
+        self.price = 0
+        self._imol = MolarFlowIndexer.from_data(
+            [streams_by_phase[i]._imol.data for i in phases], phases, 
+            chemicals=base.chemicals
+        )
+        self._sink = self._source = None
+        self.reset_cache()
+        self._register(None)
+        return self
         
     def reset_flow(self, total_flow=None, units=None, phases=None, **phase_flows):
         """
@@ -265,9 +286,9 @@ class MultiStream(Stream):
         >>> s1.reset_flow(g=[('Ethanol', 1)], phases='lgs', units='kg/hr', total_flow=2)
         >>> s1.show('cwt')
         MultiStream: s1
-         phases: ('g', 'l', 's'), T: 298.15 K, P: 101325 Pa
-         composition (%): (g) Ethanol  100
-                              -------  2 kg/hr
+        phases: ('g', 'l', 's'), T: 298.15 K, P: 101325 Pa
+        composition (%): (g) Ethanol  100
+                             -------  2 kg/hr
         
         """
         imol = self._imol
@@ -282,17 +303,20 @@ class MultiStream(Stream):
         
     def _init_indexer(self, flow, phases, chemicals, phase_flows):
         if flow == ():
+            parent = parent_indexer(chemicals)
             if phase_flows:
-                imol = MolarFlowIndexer(phases, chemicals=chemicals, **phase_flows)
+                imol = MolarFlowIndexer(phases, chemicals=chemicals, parent=parent, **phase_flows)
             else:
-                imol = MolarFlowIndexer.blank(phases, chemicals)
+                imol = MolarFlowIndexer.blank(phases, chemicals, parent)
         else:
             assert not phase_flows, ("may specify either 'flow' or "
                                     "'phase_flows', but not both")
             if isinstance(flow, MolarFlowIndexer):
                 imol = flow
             else:
-                imol = MolarFlowIndexer.from_data(flow, phases, chemicals)
+                parent = parent_indexer(chemicals)
+                imol = parent.to_material_indexer(phases)
+                imol.data[:] = flow
         self._imol = imol
     
     def _get_property(self, name, flow=False, nophase=False):
@@ -306,14 +330,17 @@ class MultiStream(Stream):
         else:
             composition = data / total
             composition_key = [i.dct for i in composition.rows]
+            T = thermal_condition._T
+            P = thermal_condition._P
             if nophase:
-                literal = (thermal_condition._T, thermal_condition._P)
+                literal = (T, P)
             else:
-                literal = (imol._phases, thermal_condition._T, thermal_condition._P)
+                phases = imol._phases
+                literal = (phases, T, P)
             last_literal, last_composition_key = self._property_cache_key
             if literal == last_literal and (composition_key == last_composition_key):
                 if name in property_cache:
-                    value = property_cache.get(name)
+                    value = property_cache[name]
                     return value * total if flow else value
             else:
                 property_cache.clear()
@@ -326,7 +353,7 @@ class MultiStream(Stream):
             else:
                 calculate = getattr(self.mixture, 'x' + name)
                 self._property_cache[name] = value = calculate(
-                    zip(imol._phases, composition), *self.thermal_condition
+                    zip(phases, composition), *self.thermal_condition
                 )
             return value * total if flow else value
     
@@ -339,32 +366,32 @@ class MultiStream(Stream):
             self._streams = {}
         self._vle_cache = eq.VLECache(self._imol,
                                       self._thermal_condition, 
-                                      self._thermo,
-                                      self._bubble_point_cache,
-                                      self._dew_point_cache)
+                                      self._thermo)
         self._lle_cache = eq.LLECache(self._imol,
                                       self._thermal_condition, 
                                       self._thermo)
         self._sle_cache = eq.SLECache(self._imol,
                                       self._thermal_condition, 
                                       self._thermo)
+        self._vlle_cache = eq.VLLECache(self._imol,
+                                        self._thermal_condition, 
+                                        self._thermo)
         
     def __getitem__(self, phase):
         streams = self._streams
+        if not (isinstance(phase, str) and len(phase) == 1):
+            raise ValueError(f'invalid phase index {phase!r}')
         if phase in streams:
             stream = streams[phase]
         else:
             stream = Stream.__new__(Stream)
             stream._ID = stream._sink = stream._source = None
-            stream._imol = self._imol.get_phase(phase)
+            stream._imol = self._imol.get_phase(phase, lock=True)
             stream._thermal_condition = self._thermal_condition
             stream._thermo = self._thermo
-            stream._bubble_point_cache =  self._bubble_point_cache
-            stream._dew_point_cache = self._dew_point_cache
             stream._property_cache = {}
             stream.characterization_factors = {}
             stream._property_cache_key = None, None
-            stream._user_equilibrium = None
             streams[phase] = stream
         return stream
     
@@ -397,6 +424,9 @@ class MultiStream(Stream):
         >>> s1 = tmo.MultiStream('s1', l=[('Water', 20), ('Ethanol', 10)], units='kg/hr')
         >>> s1.get_flow('kg/hr', ('l', 'Water'))
         20.0
+        
+        >>> s1.get_flow('kg/hr')
+        sparse([20., 10.])
 
         """
         name, factor = self._get_flow_name_and_factor(units)
@@ -441,12 +471,11 @@ class MultiStream(Stream):
         return self._imol._phases
     @phases.setter
     def phases(self, phases):
-        if len(phases) == 1:
+        phases_set = set(phases)
+        if len(phases_set) == 1:
             self.phase = phases[0]
-        phases = phase_tuple(phases)
-        if phases != self.phases:
-            self._imol = self._imol.to_material_indexer(phases)
-            self.reset_cache()
+        else:
+            self._imol = self._imol.to_material_indexer(phases_set)
     
     ### Flow properties ###
             
@@ -509,7 +538,11 @@ class MultiStream(Stream):
     @property
     def vapor_fraction(self) -> float:
         """Molar vapor fraction."""
-        return self.imol['g'].sum() / F_mol if 'g' in self.phases and (F_mol:=self.F_mol) != 0. else 0.
+        return (
+            self.imol['g'].sum() / F_mol
+            if ('g' in self.phases and (F_mol:=self.F_mol) != 0)
+            else 0.
+        )
         
     @property
     def liquid_fraction(self) -> float:
@@ -540,15 +573,15 @@ class MultiStream(Stream):
         >>> s.split_to(s1, s2, split)
         >>> s1.show(flow='kg/hr')
         MultiStream: s1
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-         flow (kg/hr): (l) Water    10
-                           Ethanol  1
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        flow (kg/hr): (l) Water    10
+                          Ethanol  1
         
         >>> s2.show(flow='kg/hr')
         MultiStream: s2
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-         flow (kg/hr): (l) Water    10
-                           Ethanol  9
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        flow (kg/hr): (l) Water    10
+                          Ethanol  9
         
         
         """
@@ -582,8 +615,8 @@ class MultiStream(Stream):
         >>> s1.copy_like(s2)
         >>> s1.show(flow='kg/hr')
         MultiStream: s1
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-         flow (kg/hr): (l) Water  2
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        flow (kg/hr): (l) Water  2
          
         Copy data from another stream with a different property package:
         
@@ -595,19 +628,11 @@ class MultiStream(Stream):
         >>> s1.copy_like(s2)
         >>> s1.show(flow='kg/hr')
         MultiStream: s1
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-         flow (kg/hr): (l) Water  2
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        flow (kg/hr): (l) Water  2
 
         """
-        if isinstance(other, MultiStream):
-            self.phases = other.phases
-            imol = other._imol
-        elif other.phase not in self.phases:
-            self.phases = (other.phase, *self.phases)
-            imol = other._imol
-        else:
-            imol = other._imol
-        self._imol.copy_like(imol)
+        self._imol.copy_like(other._imol)
         self._thermal_condition.copy_like(other._thermal_condition)
     
     def copy_flow(self, 
@@ -648,9 +673,9 @@ class MultiStream(Stream):
         >>> s2.copy_flow(s1)
         >>> s2.show(flow='kg/hr')
         MultiStream: s2
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-         flow (kg/hr): (l) Water    20
-                           Ethanol  10
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        flow (kg/hr): (l) Water    20
+                          Ethanol  10
         
         Reset and copy just water flow:
         
@@ -658,8 +683,8 @@ class MultiStream(Stream):
         >>> s2.copy_flow(s1, IDs='Water')
         >>> s2.show(flow='kg/hr')
         MultiStream: s2
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-         flow (kg/hr): (l) Water  20
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        flow (kg/hr): (l) Water  20
         
         Reset and copy all flows except water:
         
@@ -667,20 +692,20 @@ class MultiStream(Stream):
         >>> s2.copy_flow(s1, IDs='Water', exclude=True)
         >>> s2.show(flow='kg/hr')
         MultiStream: s2
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-         flow (kg/hr): (l) Ethanol  10
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        flow (kg/hr): (l) Ethanol  10
         
         Cut and paste flows:
         
         >>> s2.copy_flow(s1, remove=True)
         >>> s2.show(flow='kg/hr')
         MultiStream: s2
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-         flow (kg/hr): (l) Water    20
-                           Ethanol  10
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        flow (kg/hr): (l) Water    20
+                          Ethanol  10
         >>> s1.show()
         MultiStream: s1
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
          flow: 0
          
         The other stream can also be a single phase stream (doesn't have to be a MultiStream object):
@@ -697,9 +722,9 @@ class MultiStream(Stream):
         >>> s2.copy_flow(s1)
         >>> s2.show(flow='kg/hr')
         MultiStream: s2
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-         flow (kg/hr): (l) Water    20
-                           Ethanol  10
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        flow (kg/hr): (l) Water    20
+                          Ethanol  10
         
         Reset and copy just water flow:
         
@@ -707,8 +732,8 @@ class MultiStream(Stream):
         >>> s2.copy_flow(s1, IDs='Water')
         >>> s2.show(flow='kg/hr')
         MultiStream: s2
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-         flow (kg/hr): (l) Water  20
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        flow (kg/hr): (l) Water  20
         
         Reset and copy all flows except water:
         
@@ -716,36 +741,35 @@ class MultiStream(Stream):
         >>> s2.copy_flow(s1, IDs='Water', exclude=True)
         >>> s2.show(flow='kg/hr')
         MultiStream: s2
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-         flow (kg/hr): (l) Ethanol  10
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        flow (kg/hr): (l) Ethanol  10
         
         Cut and paste flows:
         
         >>> s2.copy_flow(s1, remove=True)
         >>> s2.show(flow='kg/hr')
         MultiStream: s2
-         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
-         flow (kg/hr): (l) Water    20
-                           Ethanol  10
+        phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+        flow (kg/hr): (l) Water    20
+                          Ethanol  10
         >>> s1.show()
         Stream: s1
-         phase: 'l', T: 298.15 K, P: 101325 Pa
-         flow: 0
+        phase: 'l', T: 298.15 K, P: 101325 Pa
+        flow: 0
         
         """
-        if self.chemicals is not other.chemicals:
+        if self.chemicals is not other.chemicals and self.chemicals.IDs != other.chemicals.IDs:
             raise ValueError('other stream must have the same chemicals defined to copy flow')
-        other_ismultistream = isinstance(other, MultiStream)
-        if other_ismultistream: self.phases = [*self.phases, *other.phases]
         IDs_index = self.chemicals.get_index(IDs)
         phase_index = self.imol.get_phase_index(phase)
         data = self.imol.data
         other_data = other.imol.data
+        multiphase = other_data.ndim == 2
         if exclude:
             data = self.imol.data
             other_data = other.imol.data
             original_data = data.copy()
-            if other_ismultistream:
+            if multiphase:
                 data[:] = other_data
                 data[phase_index, IDs_index] = original_data[phase_index, IDs_index]
                 if remove:
@@ -760,7 +784,7 @@ class MultiStream(Stream):
                     excluded_data = other_data[IDs_index]
                     other_data[:] = 0.
                     other_data[IDs_index] = excluded_data   
-        elif other_ismultistream:
+        elif multiphase:
             data[phase_index, IDs_index] = other_data[phase_index, IDs_index]
             if remove: other_data[phase_index, IDs_index] = 0.
         else:
@@ -912,10 +936,10 @@ class MultiStream(Stream):
         >>> tmo.settings.set_thermo(['Water', 'Ethanol', 'Methanol'], cache=True) 
         >>> s1 = tmo.MultiStream('s1', l=[('Water', 20), ('Ethanol', 10), ('Methanol', 10)], units='m3/hr')
         >>> s1.get_concentration('l', ('Water', 'Ethanol'))
-        array([27.672,  4.265])
+        array([27.673,  4.261])
 
         >>> s1.get_concentration('l', ('Water', 'Ethanol'), 'g/L')
-        array([498.512, 196.479])
+        array([498.532, 196.291])
 
         """
         F_vol = self.F_vol
@@ -947,24 +971,23 @@ class MultiStream(Stream):
     @property
     def vle(self) -> eq.VLE:
         """An object that can perform vapor-liquid equilibrium on the stream."""
-        phases = self.phases
-        if 'l' not in phases or 'g' not in phases: 
-            self.phases = [*phases, 'l', 'g']
+        self._imol._expand_phases('lg')
         return self._vle_cache.retrieve()
     @property
     def lle(self) -> eq.LLE:
         """An object that can perform liquid-liquid equilibrium on the stream."""
-        phases = self.phases
-        if 'l' not in phases or 'L' not in phases: 
-            self.phases = [*phases, 'l', 'L']
+        self._imol._expand_phases('Ll')
         return self._lle_cache.retrieve()
     @property
     def sle(self) -> eq.SLE:
         """An object that can perform solid-liquid equilibrium on the stream."""
-        phases = self.phases
-        if 's' not in phases or 'l' not in phases: 
-            self.phases = [*phases, 's', 'l']
+        self._imol._expand_phases('sl')
         return self._sle_cache.retrieve()
+    @property
+    def vlle(self) -> eq.VLLE:
+        """An object that can perform vapor-liquid-liquid equilibrium on the stream."""
+        self._imol._expand_phases('Llg')
+        return self._vlle_cache.retrieve()
     
     ### Casting ###
     
@@ -979,25 +1002,33 @@ class MultiStream(Stream):
         else:
             raise RuntimeError('multiple phases present; cannot convert to single phase stream')
     
+    def reduce_phases(self):
+        """Remove empty phases."""
+        self.phase = self.phase
+    
     @property
     def phase(self) -> str:
         imol = self._imol
-        return ''.join([phases[0] for phases in ('g', 'lL', 'sS') if not imol.phases_are_empty(phases)])
+        return ''.join([i for i, j in imol if j.any()])
     @phase.setter
     def phase(self, phase):
-        if len(phase) > 1: 
+        N_phase = len(phase)
+        m = self.mol.copy()
+        if N_phase > 1: 
             self.phases = phase
         else:
+            if N_phase == 0: phase = 'l' # Default phase for streams
             self._imol = self._imol.to_chemical_indexer(phase)
-            self._streams.clear()
             self.__class__ = Stream
     
     ### Representation ###
     
-    def _info_str(self, T_units, P_units, flow_units, composition, N_max, all_IDs, indexer, factor):
+    def _info_str(self, units, notation, composition, N_max, all_IDs, indexer, factor):
         """Return string with all specifications."""
         basic_info = self._basic_info()
-        basic_info += Stream._info_phaseTP(self, self.phases, T_units, P_units)
+        basic_info += Stream._info_phaseTP(self, self.phases, units, notation)
+        flow_units = units['flow']
+        flow_notation = notation['flow']
         N_all_IDs = len(all_IDs)
         if N_all_IDs == 0:
             return basic_info + ' flow: 0' 
@@ -1007,9 +1038,9 @@ class MultiStream(Stream):
         maxlen = max(all_lengths) + 2
         
         if composition:
-            first_line = " composition (%):"
+            first_line = "composition (%):"
         else:
-            first_line = f' flow ({flow_units}):'
+            first_line = f'flow ({flow_units}):'
         first_line_spaces = len(first_line)*" "
 
         # Set up chemical data for all phases
@@ -1037,13 +1068,13 @@ class MultiStream(Stream):
             for i in range(N):
                 spaces = ' ' * (maxlen - lengths[i])
                 if i: flow_rates += new_line    
-                flow_rates += f'{IDs[i]}' + spaces + f'{data[i]:.3g}'
+                flow_rates += f'{IDs[i]}' + spaces + f'{data[i]:{flow_notation}}'
             if too_many_chemicals:
                 spaces = ' ' * (maxlen - 3)
-                flow_rates += new_line + '...' + spaces + f'{data[N_max:].sum():.3g}'
+                flow_rates += new_line + '...' + spaces + f'{data[N_max:].sum():{flow_notation}}'
             if composition:
                 dashes = '-' * (maxlen - 2)
-                flow_rates += f"{new_line}{dashes}  {total_flow:.3g} {flow_units}"
+                flow_rates += f"{new_line}{dashes}  {total_flow:{flow_notation}} {flow_units}"
             # Put it together
             phases_flow_rates_info += beginning + flow_rates + '\n'
             
