@@ -9,8 +9,7 @@
 """
 from .exceptions import UndefinedPhase
 
-__all__ = ('Phase', 'LockedPhase', 'NoPhase', 'PhaseIndexer',
-           'phase_tuple', 'valid_phases')
+__all__ = ('PhaseIndexer', 'phase_tuple', 'valid_phases')
 
 new = object.__new__
 valid_phases = {
@@ -20,6 +19,7 @@ valid_phases = {
     'S': 'SOLID',
     'L': 'LIQUID',
 }
+index = {j: i for i,j in enumerate(sorted(valid_phases))}
 
 def check_phase(phase):
     """
@@ -33,11 +33,12 @@ def check_phase(phase):
     's' (solid), 'l' (liquid), 'g' (gas), 'S' (SOLID), and 'L' (LIQUID)
     
     """
+    if phase is None: return
     if phase not in valid_phases:
         raise RuntimeError(
             f"invalid phase {repr(phase)} encountered; valid phases are "
             "'s' (solid), 'l' (liquid), 'g' (gas), 'S' (SOLID), and 'L' (LIQUID)"
-        )  
+        )
 
 def phase_tuple(phases):
     """
@@ -53,7 +54,7 @@ def phase_tuple(phases):
     ('g', 'l', 's')
     
     """
-    phases = set(phases)
+    phases = frozenset(phases)
     for i in phases: check_phase(i)
     return tuple(sorted(phases))
 
@@ -92,7 +93,7 @@ class PhaseIndexer:
     True
     
     """
-    __slots__ = ('_index',)
+    __slots__ = ('_index', '_phases', '_compatibility')
     _index_cache = {}
     
     def __new__(cls, phases):
@@ -102,75 +103,37 @@ class PhaseIndexer:
             self = cache[phases]
         else:
             cache[phases] = self = new(cls)
-            self._index = index = {j:i for i,j in enumerate(sorted(phases))}
-            index[...] = slice(None) 
+            self._phases = phases = tuple(sorted(phases))
+            self._index = index = {j: i for i,j in enumerate(phases)}
+            for phase, n in tuple(index.items()):
+                if phase.isupper():
+                    phase = phase.lower()
+                else:
+                    phase = phase.upper()
+                if phase not in index: index[phase] = n
+            self._compatibility = ''.join([i.lower() for i in phases])
+            index[...] = slice(None)
         return self
     
+    @property
+    def phases(self):
+        return self._phases
+    
+    def __contains__(self, phase):
+        return phase in self._index
+    
+    def compatible_with(self, other):
+        return self._compatibility == other._compatibility
+    
     def __call__(self, phase):
         try:
             return self._index[phase]
         except:
             raise UndefinedPhase(phase)        
     
-    @property
-    def phases(self):
-        return tuple(self._index)[:-1]
-    
     def __reduce__(self):
-        return PhaseIndexer, (self.phases,)
+        return PhaseIndexer, (self._phases,)
     
     def __repr__(self):
-        return f"{type(self).__name__}({list(self.phases)})"
+        return f"{type(self).__name__}({[*self._phases]})"
 
-class Phase:
-    __slots__ = ('_phase',)
-    
-    @classmethod
-    def convert(cls, phase):
-        return phase if isinstance(phase, cls) else cls(phase)
-    
-    def __new__(cls, phase):
-        self = new(cls)
-        self._phase = phase
-        return self
-    
-    def __reduce__(self):
-        return Phase, (self.phase,)
-    
-    @property
-    def phase(self):
-        return self._phase
-    @phase.setter
-    def phase(self, phase):
-        check_phase(phase)
-        self._phase = phase
-    
-    def copy(self):
-        return Phase(self.phase)
-    __copy__ = copy
-    
-    def __repr__(self):
-        return f"{type(self).__name__}({repr(self.phase)})"
-
-
-class LockedPhase(Phase):
-    __slots__ = ()
-    _cache = {}
-    
-    def __new__(cls, phase):
-        cache = cls._cache
-        if phase in cache:
-            self = cache[phase]
-        else:
-            cache[phase] = self = new(cls)
-            object.__setattr__(self, '_phase', phase)
-        return self
-    
-    def __reduce__(self):
-        return Phase, (self.phase,)
-    
-    def __setattr__(self, name, value):
-        if value != self.phase:
-            raise AttributeError('phase is locked')
-        
-NoPhase = LockedPhase(None)
