@@ -12,7 +12,7 @@ import thermosteam as tmo
 from . import equilibrium as eq
 from ._chemical import Chemical
 from ._chemicals import Chemicals
-from .mixture import Mixture
+from .mixture import Mixture, IdealMixture
 from .utils import read_only, cucumber
 
 __all__ = ('Thermo', 'IdealThermo')
@@ -35,10 +35,12 @@ class Thermo:
         Class for computing fugacity coefficients.
     PCF : :class:`~thermosteam.equilibrium.poyinting_correction_factors.PoyintingCorrectionFactors` subclass, optional
         Class for computing poynting correction factors.
-    cache : optional
+    cache : bool, optional
         Whether or not to use cached chemicals.
     skip_checks : bool, optional
         Whether to skip checks for missing or invalid properties.
+    db : str, optional
+        Database to load any chemicals.
     
     Examples
     --------
@@ -49,8 +51,7 @@ class Thermo:
     >>> thermo.show() 
     Thermo(
         chemicals=CompiledChemicals([Ethanol, Water]),
-        mixture=Mixture(
-            rule='ideal', ...
+        mixture=IdealMixture(...
             include_excess_energies=False
         ),
         Gamma=DortmundActivityCoefficients,
@@ -66,8 +67,7 @@ class Thermo:
     >>> ideal.show()
     IdealThermo(
         chemicals=CompiledChemicals([Ethanol, Water]),
-        mixture=Mixture(
-            rule='ideal', ...
+        mixture=IdealMixture(...
             include_excess_energies=False
         ),
     )
@@ -80,20 +80,20 @@ class Thermo:
     >>> stream.vle(T=361, P=101325)
     >>> stream.show()
     MultiStream: stream
-     phases: ('g', 'l'), T: 361 K, P: 101325 Pa
-     flow (kmol/hr): (g) Ethanol  32.6
-                         Water    17.6
-                     (l) Ethanol  67.4
-                         Water    82.4
+    phases: ('g', 'l'), T: 361 K, P: 101325 Pa
+    flow (kmol/hr): (g) Ethanol  30
+                        Water    16
+                    (l) Ethanol  70
+                        Water    84
     >>> # Modified Roult's law:                 
     >>> tmo.settings.set_thermo(thermo)
     >>> stream = tmo.Stream('stream', Water=100, Ethanol=100)
-    >>> stream.vle(T=360, P=101325) 
+    >>> stream.vle(T=361, P=101325) 
     >>> stream.show()
     MultiStream: stream
-     phases: ('g', 'l'), T: 360 K, P: 101325 Pa
-     flow (kmol/hr): (g) Ethanol  100
-                         Water    100
+    phases: ('g', 'l'), T: 361 K, P: 101325 Pa
+    flow (kmol/hr): (g) Ethanol  100
+                        Water    100
     
     Thermodynamic property packages are pickleable:
         
@@ -102,8 +102,7 @@ class Thermo:
     >>> thermo.show()
     Thermo(
         chemicals=CompiledChemicals([Ethanol, Water]),
-        mixture=Mixture(
-            rule='ideal', ...
+        mixture=IdealMixture(...
             include_excess_energies=False
         ),
         Gamma=DortmundActivityCoefficients,
@@ -132,13 +131,14 @@ class Thermo:
                  Phi=None,
                  PCF=None,
                  cache=None,
-                 skip_checks=False):
+                 skip_checks=False,
+                 db='default'):
         if Gamma is None: Gamma = eq.DortmundActivityCoefficients
         if Phi is None: Phi = eq.IdealFugacityCoefficients
         if PCF is None: PCF = eq.MockPoyintingCorrectionFactors
-        if not isinstance(chemicals, Chemicals): chemicals = Chemicals(chemicals, cache)
+        if not isinstance(chemicals, Chemicals): chemicals = Chemicals(chemicals, cache, db=db)
         if not mixture:
-            mixture = Mixture.from_chemicals(chemicals)
+            mixture = IdealMixture.from_chemicals(chemicals)
         elif not isinstance(mixture, Mixture): # pragma: no cover
             raise ValueError(f"mixture must be a '{Mixture.__name__}' object")
         chemicals.compile(skip_checks=skip_checks)
@@ -167,12 +167,20 @@ class Thermo:
     def __exit__(self):
         tmo.settings.set_thermo(self._original_thermo)
     
+    def extended(self, chemicals):
+        original_chemicals = self.chemicals
+        return self.subset([
+            *original_chemicals,
+            *[i for i in chemicals if i.ID not in original_chemicals],
+        ])
+    
     def subset(self, chemicals):
         if chemicals is self.chemicals: return self
-        if not isinstance(chemicals, Chemicals): chemicals = Chemicals(chemicals)
-        ndarray = np.ndarray
+        isa = isinstance
+        if not isa(chemicals, Chemicals): 
+            chemicals = Chemicals([self.as_chemical(i) for i in chemicals])
         groups = [(name, index) for name, index in self.chemicals._index.items() 
-                  if isinstance(index, ndarray)]
+                  if isa(index, list)]
         chemicals.compile(skip_checks=True)
         CASs = self.chemicals.CASs
         for name, index in groups:
@@ -190,6 +198,21 @@ class Thermo:
         setattr(new, '_original_thermo', None)
         return new
     
+    def fugacities(self, phase, chemicals):
+        p = phase.lower()
+        if p == 'g':
+            return self.gas_fugacities(chemicals)
+        elif p == 'l':
+            return self.liquid_fugacities(chemicals)
+        else:
+            raise NotImplementedError(f'fugacities for phase {phase!r} not available')
+    
+    def gas_fugacities(self, chemicals):
+        return eq.GasFugacities(chemicals, self)
+    
+    def liquid_fugacities(self, chemicals):
+        return eq.LiquidFugacities(chemicals, self)
+    
     def ideal(self):
         """Ideal thermodynamic property package."""
         ideal = self._ideal
@@ -223,12 +246,9 @@ class Thermo:
         """
         isa = isinstance
         if isa(chemical, str):
-            try: 
-                chemical = self.chemicals[chemical]
-            except:
-                chemical = Chemical(chemical)
+            chemical = self.chemicals[chemical] if chemical in self.chemicals else Chemical(chemical)
         elif not isa(chemical, Chemical): # pragma: no cover
-            raise ValueError(f"can only convert '{type(chemical).__name__}' object to chemical")
+            raise ValueError(f"cannot convert '{type(chemical).__name__}' object to chemical")
         return chemical
     
     def __repr__(self):
@@ -299,10 +319,11 @@ class IdealThermo:
     
     def subset(self, chemicals):
         if chemicals is self.chemicals: return self
-        if not isinstance(chemicals, Chemicals): chemicals = Chemicals(chemicals)
-        ndarray = np.ndarray
+        isa = isinstance
+        if not isa(chemicals, Chemicals): 
+            chemicals = Chemicals([self.as_chemical(i) for i in chemicals])
         groups = [(name, index) for name, index in self.chemicals._index.items() 
-                  if isinstance(index, ndarray)]
+                  if isinstance(index, list)]
         chemicals.compile(skip_checks=True)
         CASs = self.chemicals.CASs
         for name, index in groups:
@@ -313,7 +334,7 @@ class IdealThermo:
         setattr = object.__setattr__
         setattr(new, 'chemicals', chemicals)
         setattr(new, 'mixture', self.mixture.from_chemicals(chemicals))
-        setattr(new, '_original_thermo', None)
+        setattr(new, '_original_thermo', self._original_thermo)
         return new
     
     def ideal(self):
