@@ -39,7 +39,7 @@ class LiquidFugacities:
     >>> liquid_molar_composition = np.array([0.72, 0.28])
     >>> f_l = F_l(x=liquid_molar_composition, T=355, P=101325.)
     >>> f_l
-    array([43338.226, 58056.67 ])
+    array([43338.226, 57731.001])
     
     """
     __slots__ = ('gamma', 'chemicals', 'pcf')
@@ -50,6 +50,10 @@ class LiquidFugacities:
         self.gamma = thermo.Gamma(chemicals)
         self.pcf = thermo.PCF(chemicals)
     
+    def coefficient(self, x, T, P=101325.):
+        Psats = np.array([i.Psat(T) for i in self.chemicals], dtype=float)
+        return self.gamma(x, T) * self.pcf(T, P, Psats) * Psats
+    
     def __call__(self, x, T, P=101325.):
         Psats = np.array([i.Psat(T) for i in self.chemicals], dtype=float)
         return x * self.gamma(x, T) * self.pcf(T, P, Psats) * Psats
@@ -95,6 +99,9 @@ class GasFugacities:
         self.chemicals = chemicals = tuple(chemicals)
         self.phi = thermo.Phi(chemicals)
     
+    def coefficient(self, y, T, P):
+        return P * self.phi(y, T, P)
+    
     def __call__(self, y, T, P):
         return P * self.phi(y, T, P) * y
     
