@@ -12,7 +12,7 @@ from . import thermo_user
 from . import read_only
 from . import registered
 from . import forward
-from . import units_of_measure
+from . import define_units_of_measure
 from . import chemical_cache
 
 __all__ = (*chemicals_user.__all__,
@@ -20,7 +20,7 @@ __all__ = (*chemicals_user.__all__,
            *read_only.__all__,
            *registered.__all__,
            *forward.__all__,
-           *units_of_measure.__all__,
+           *define_units_of_measure.__all__,
            *chemical_cache.__all__)
 
 from .registered import *
@@ -28,5 +28,5 @@ from .chemicals_user import *
 from .thermo_user import *
 from .read_only import *
 from .forward import *
-from .units_of_measure import *
+from .define_units_of_measure import *
 from .chemical_cache import *
