@@ -13,21 +13,11 @@ __all__ = ('Registry', 'is_valid_ID', 'check_valid_ID')
 
 
 def is_valid_ID(ID):
-    if not isinstance(ID, str): return False
-    if not ID[0].isalpha(): return False
-    if not all([word.isalnum() for word in ID.split('_') if word]): return False
-    return True
+    return isinstance(ID, str)
 
 def check_valid_ID(ID):
     if not isinstance(ID, str):
         raise RuntimeError(f"ID must be a string, not a '{type(ID).__name__}' object")
-    if not ID[0].isalpha():
-        raise RuntimeError("ID must start with a letter")
-    if not all([word.isalnum() for word in ID.split('_') if word]):
-        raise RuntimeError(
-            'ID may only contain letters, numbers, and/or underscores; '
-            'no special characters or spaces'
-        )
         
 def check_valid_alias(alias):
     if not isinstance(alias, str):
@@ -89,62 +79,64 @@ class Registry: # pragma: no cover
     
     def register_safely(self, ID, obj):
         """Register object safely, with checks and due warnings."""
-        if '.' in ID:
-            data = self.data
-            ID_old = getattr(obj, '_ID', None)
-            if ID_old and data.get(ID_old) is obj: del data[ID_old]
-            obj._ID = ID[1:] if ID[0] == '.' else ID
-        else:
-            check_valid_ID(ID)
-            data = self.data
-            ID_old = self._open_registration(ID, obj)
-            if ID in data:
-                other = data[ID]
-                if obj is not other and other not in self.safe_to_replace:
-                    if self.AUTORENAME:
-                        try:
-                            base, num = ID.rsplit('_', 1)
-                            num = int(num)
-                        except:
-                            other.ID = ID + '_1'
-                        else:
-                            other.ID = base + '_' + str(num + 1) 
-                        base, num = other.ID.rsplit('_', 1)
-                        ID = base + '_' + str(int(num) + 1)
-                    elif ID_old:
-                        warning = RuntimeWarning(
-                            f"upon renaming, {repr(obj)} replaced {repr(other)} "
-                             "in registry"
-                        )
-                        warn(warning, 4)
+        check_valid_ID(ID)
+        data = self.data
+        ID_old = self._open_registration(ID, obj)
+        if ID in data:
+            other = data[ID]
+            if obj is not other and other not in self.safe_to_replace:
+                if self.AUTORENAME:
+                    try:
+                        base, num = ID.rsplit('_', 1)
+                        num = int(num)
+                    except:
+                        other.ID = ID + '_1'
                     else:
-                        warning = RuntimeWarning(
-                            f"{repr(other)} has been replaced in registry"
-                        )
-                        warn(warning, stacklevel=getattr(obj, '_stacklevel', 5) - 1)
-            self._close_registration(ID, obj)
+                        other.ID = base + '_' + str(num + 1) 
+                    base, num = other.ID.rsplit('_', 1)
+                    ID = base + '_' + str(int(num) + 1)
+                elif ID_old:
+                    warning = RuntimeWarning(
+                        f"upon renaming, {obj!r} replaced {other!r} "
+                         "in registry"
+                    )
+                    warn(warning, 4)
+                else:
+                    warning = RuntimeWarning(
+                        f"{other!r} has been replaced in registry"
+                    )
+                    warn(warning, stacklevel=getattr(obj, '_stacklevel', 4) - 1)
+        self._close_registration(ID, obj)
         
     def register(self, ID, obj):
         """Register object without warnings or checks."""
         self._open_registration(ID, obj)
         self._close_registration(ID, obj)
         
-    def register_alias_safely(self, alias, obj):
+    def register_alias_safely(self, alias, obj, override=None):
         """Register object safely, with checks and due warnings."""
         check_valid_alias(alias)
         data = self.data
         if alias in data:
             other = data[alias]
             if obj is not other and other not in self.safe_to_replace:
-                warning = RuntimeWarning(
-                    f"alias {repr(alias)} of {repr(other)} has been replaced in registry with {repr(obj)}"
-                )
-                warn(warning, stacklevel=getattr(obj, '_stacklevel', 5) - 1)
+                if override is None:
+                    warning = RuntimeWarning(
+                        f"alias {alias!r} of {other!r} has been replaced in registry with {obj!r}"
+                    )
+                    warn(warning, stacklevel=getattr(obj, '_stacklevel', 5) - 1)
+                elif not override:
+                    raise RuntimeError(
+                        f"alias {alias!r} is already registered to {other!r}"
+                    )
         self.register_alias(alias, obj)
         
-    def register_alias(self, alias, obj):
+    def register_alias(self, alias, obj, override=True):
         """Register object alias without warnings or checks."""
-        self.data[alias] = obj
+        if override:
+            self.data[alias] = obj
+        elif alias not in self.data:
+            self.data[alias] = obj
         for i in self.context_levels: i.append(obj)
         
     def _open_registration(self, ID, obj):
@@ -230,9 +222,9 @@ class Registry: # pragma: no cover
     
     def show(self):
         if self.data:
-            print('Register:\n ' + '\n '.join([repr(i) for i in self]))
+            print(f'{type(self).__name__}:\n' + '\n'.join([repr(i) for i in self]))
         else:
-            print('Register: (Empty)')
+            print(f'{type(self).__name__}: (Empty)')
     
     _ipython_display_ = show
 
